package at.gv.brz.bjujq.erv.samples.common;

import at.gv.justiz.erv.ervanschriftcodenachrichtextern.v3_0.AnschriftcodedatenKopplungTyp;
import at.gv.justiz.erv.ervanschriftcodenachrichtextern.v3_0.AnschriftcodedatenTyp;
import at.gv.justiz.erv.ervanschriftcodeserviceextern.v3_0.*;
import at.gv.justiz.erv.ervwertebereichextern.v3_0.AnschriftcodeStatus;
import at.gv.justiz.erv.ervwertebereichextern.v3_0.KopplungsStatus;
import at.gv.justiz.erv.ervwertebereichextern.v3_0.Service;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * Simple ERV-Anschriftcode port mock builder.
 */
public final class MockErvAnschriftcodePortBuilder {

    private ERVAnschriftcodePortType mock;

    public MockErvAnschriftcodePortBuilder() {
        this.mock = mock(ERVAnschriftcodePortType.class);
    }

    public ERVAnschriftcodePortType build() {
        return this.mock;
    }

    public MockErvAnschriftcodePortBuilder withAnschriftcodeAbfragenKopplung() {
        try {
            when(this.mock.anschriftcodeAbfragenKopplung(any(AnschriftcodeAbfragenRequestKopplung.class))).then(new Answer<AnschriftcodeAbfragenResponseKopplung>() {
                @Override
                public AnschriftcodeAbfragenResponseKopplung answer(InvocationOnMock invocation) throws Throwable {

                    AnschriftcodeAbfragenRequestKopplung request = (AnschriftcodeAbfragenRequestKopplung) invocation.getArgument(0);
                    SchemaUtil.printWithHeader("AnschriftcodeAbfragenKopplung Request", request);

                    AnschriftcodeAbfragenResponseKopplung response = SchemaUtil.OF_AN_SERVICE.createAnschriftcodeAbfragenResponseKopplung();

                    for (String anschriftcode : request.getAnschriftcode()) {
                        AnschriftcodedatenKopplungTyp anschriftcodedatenTyp = SchemaUtil.OF_AN_NR.createAnschriftcodedatenKopplungTyp();
                        anschriftcodedatenTyp.setAnschriftcode(anschriftcode);
                        anschriftcodedatenTyp.setService(SchemaUtil.OF_TYP.createServiceTyp());
                        anschriftcodedatenTyp.getService().setServiceProvider("TEST");
                        anschriftcodedatenTyp.getService().setServiceArt(Service.UEBERMITTLUNGSSTELLE);
                        anschriftcodedatenTyp.setStatus(AnschriftcodeStatus.R);
                        anschriftcodedatenTyp.setZeitpunkt(SchemaUtil.createCurrentDateTime());
                        anschriftcodedatenTyp.setKopplungsStatus(KopplungsStatus.K);
                        anschriftcodedatenTyp.setKopplungsZustelldienst("TEST");
                        response.getAnschriftcodedatenKopplung().add(anschriftcodedatenTyp);
                    }

                    SchemaUtil.printWithHeader("AnschriftcodeAbfragenKopplung Response", response);
                    return response;
                }
            });
        } catch (ERVAnschriftcodeServiceException_Exception e) {
            throw new RuntimeException(e);
        }

        return this;
    }

    public MockErvAnschriftcodePortBuilder withStatusSetzen() {
        try {
            when(this.mock.statusSetzen(any(StatusSetzenRequest.class))).then(new Answer<StatusSetzenResponse>() {
                @Override
                public StatusSetzenResponse answer(InvocationOnMock invocation) throws Throwable {
                    StatusSetzenRequest request = (StatusSetzenRequest) invocation.getArgument(0);
                    SchemaUtil.printWithHeader("StatusSetzen Request", request);

                    StatusSetzenResponse response = SchemaUtil.OF_AN_SERVICE.createStatusSetzenResponse();

                    SchemaUtil.printWithHeader("StatusSetzen Response", response);

                    return response;
                }
            });
        } catch (ERVAnschriftcodeServiceException_Exception e) {
            throw new RuntimeException(e);
        }

        return this;
    }

    public MockErvAnschriftcodePortBuilder withAnschriftcodeAbfragen() {
        try {
            when(this.mock.anschriftcodeAbfragen(any(AnschriftcodeAbfragenRequest.class))).then(new Answer<AnschriftcodeAbfragenResponse>() {
                @Override
                public AnschriftcodeAbfragenResponse answer(InvocationOnMock invocation) throws Throwable {

                    AnschriftcodeAbfragenRequest request = (AnschriftcodeAbfragenRequest) invocation.getArgument(0);
                    SchemaUtil.printWithHeader("AnschriftcodeAbfragen Request", request);

                    AnschriftcodeAbfragenResponse response = SchemaUtil.OF_AN_SERVICE.createAnschriftcodeAbfragenResponse();

                    for (String anschriftcode : request.getAnschriftcode()) {
                        AnschriftcodedatenTyp anschriftcodedatenTyp = SchemaUtil.OF_AN_NR.createAnschriftcodedatenTyp();
                        anschriftcodedatenTyp.setAnschriftcode(anschriftcode);
                        anschriftcodedatenTyp.setService(SchemaUtil.OF_TYP.createServiceTyp());
                        anschriftcodedatenTyp.getService().setServiceProvider("TEST");
                        anschriftcodedatenTyp.getService().setServiceArt(Service.UEBERMITTLUNGSSTELLE);
                        anschriftcodedatenTyp.setStatus(AnschriftcodeStatus.R);
                        anschriftcodedatenTyp.setZeitpunkt(SchemaUtil.createCurrentDateTime());
                        response.getAnschriftcodedaten().add(anschriftcodedatenTyp);
                    }

                    SchemaUtil.printWithHeader("AnschriftcodeAbfragen Response", response);

                    return response;
                }
            });
        } catch (ERVAnschriftcodeServiceException_Exception e) {
            throw new RuntimeException(e);
        }

        return this;
    }
}
