package at.gv.brz.bjujq.erv.samples.common;

import at.gv.justiz.erv.ervnachrichtserviceextern.v3_0.*;
import at.gv.justiz.erv.ervwertebereichextern.v3_0.NachrichtStatuscode;
import org.mockito.Mockito;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;

/**
 * Simple ERV port mock builder.
 */
public final class MockErvNachrichtPortBuilder {

    private ERVNachrichtPortType mock;

    public MockErvNachrichtPortBuilder() {
        this.mock = Mockito.mock(ERVNachrichtPortType.class);
    }

    public ERVNachrichtPortType build() {
        return this.mock;
    }

    public MockErvNachrichtPortBuilder withNachweisAbholen(String anschriftcode, String justizanwendung) {
        try {
            when(this.mock.nachweisAbholen(any(NachweisAbholenRequest.class))).then(new Answer<NachweisAbholenResponse>() {
                @Override
                public NachweisAbholenResponse answer(InvocationOnMock invocation) throws Throwable {

                    final NachweisAbholenRequest request = (NachweisAbholenRequest) invocation.getArgument(0);
                    SchemaUtil.printWithHeader("NachweisAbholen Request", request);

                    final NachweisAbholenResponse response = SchemaUtil.OF_SERVICE.createNachweisAbholenResponse();
                    response.setERVNachricht(SchemaUtil.createErvNachweisFromJustizAT(anschriftcode, justizanwendung, request.getNachrichtenkennung().getNachrichtId()));

                    SchemaUtil.printWithHeader("NachweisAbholen Response", response);
                    return response;
                }
            });
        } catch (ERVException_Exception e) {
            throw new RuntimeException(e);
        }

        return this;
    }

    public MockErvNachrichtPortBuilder withNachweisEinbringen() {
        try {
            when(this.mock.nachweisEinbringen(any(NachweisEinbringenRequest.class))).then(new Answer<NachweisEinbringenResponse>() {
                @Override
                public NachweisEinbringenResponse answer(InvocationOnMock invocation) throws Throwable {

                    NachweisEinbringenRequest request = (NachweisEinbringenRequest) invocation.getArgument(0);
                    SchemaUtil.printWithHeader("NachweisEinbringen Request", request);

                    final NachweisEinbringenResponse response = SchemaUtil.OF_SERVICE.createNachweisEinbringenResponse();
                    response.setNachrichtstatus(SchemaUtil.createStatus(NachrichtStatuscode.HINTERLEGT));

                    SchemaUtil.printWithHeader("NachweisEinbringen Response", response);
                    return response;
                }
            });
        } catch (ERVException_Exception e) {
            throw new RuntimeException(e);
        }

        return this;
    }

    public MockErvNachrichtPortBuilder withNachrichtEinbringen() {
        try {
            when(this.mock.nachrichtEinbringen(any(NachrichtEinbringenRequest.class))).then(new Answer<NachrichtEinbringenResponse>() {
                @Override
                public NachrichtEinbringenResponse answer(InvocationOnMock invocation) throws Throwable {

                    NachrichtEinbringenRequest request = (NachrichtEinbringenRequest) invocation.getArgument(0);
                    SchemaUtil.printWithHeader("NachrichtEinbringen Request", request);

                    final NachrichtEinbringenResponse response = SchemaUtil.OF_SERVICE.createNachrichtEinbringenResponse();
                    response.setNachrichtstatus(SchemaUtil.createStatus(NachrichtStatuscode.HINTERLEGT));

                    SchemaUtil.printWithHeader("NachrichtEinbringen Response", response);
                    return response;
                }
            });
        } catch (ERVException_Exception e) {
            throw new RuntimeException(e);
        }

        return this;
    }

    public MockErvNachrichtPortBuilder withNachrichtStatus() {
        try {
            when(this.mock.nachrichtStatus(any(NachrichtStatusRequest.class))).then(new Answer<NachrichtStatusResponse>() {
                @Override
                public NachrichtStatusResponse answer(InvocationOnMock invocationOnMock) throws Throwable {

                    NachrichtStatusRequest request = (NachrichtStatusRequest) invocationOnMock.getArgument(0);
                    SchemaUtil.printWithHeader("NachrichtStatus Request", request);

                    NachrichtStatusResponse response = SchemaUtil.OF_SERVICE.createNachrichtStatusResponse();
                    response.setNachrichtstatus(SchemaUtil.OF_NR.createStatusTyp());
                    response.getNachrichtstatus().setZeitpunkt(SchemaUtil.createCurrentDateTime());
                    response.getNachrichtstatus().setCode(NachrichtStatuscode.ZUGESTELLT);
                    response.setNachweisVorhanden(true);
                    SchemaUtil.printWithHeader("NachrichtStatus Response", response);

                    return response;
                }
            });
        } catch (ERVException_Exception e) {
            throw new RuntimeException(e);
        }

        return this;
    }

    public MockErvNachrichtPortBuilder withNachrichtBestaetigen() {
        try {
            when(this.mock.nachrichtBestaetigen(any(NachrichtBestaetigenRequest.class))).then(new Answer<NachrichtBestaetigenResponse>() {
                @Override
                public NachrichtBestaetigenResponse answer(InvocationOnMock invocation) throws Throwable {

                    NachrichtBestaetigenRequest request = (NachrichtBestaetigenRequest) invocation.getArgument(0);
                    SchemaUtil.printWithHeader("NachrichtBestaetigen Request", request);

                    NachrichtBestaetigenResponse response = SchemaUtil.OF_SERVICE.createNachrichtBestaetigenResponse();
                    response.setNachrichtstatus(SchemaUtil.createStatus(NachrichtStatuscode.ZUGESTELLT));

                    SchemaUtil.printWithHeader("NachrichtBestaetigen Response", response);

                    return response;
                }
            });
        } catch (ERVException_Exception e) {
            throw new RuntimeException(e);
        }

        return this;
    }

    public MockErvNachrichtPortBuilder withNachrichtAbholen(final String anschriftcode, final String justizanwendung) {
        try {
            when(this.mock.nachrichtAbholen(any())).then(new Answer<NachrichtAbholenResponse>() {
                @Override
                public NachrichtAbholenResponse answer(InvocationOnMock invocation) throws Throwable {
                    final NachrichtAbholenResponse response = SchemaUtil.OF_SERVICE.createNachrichtAbholenResponse();
                    response.setERVNachricht(SchemaUtil.createErvNachrichtFromJustizAT(anschriftcode, justizanwendung));

                    SchemaUtil.printWithHeader("NachrichtAbholen Response", response);
                    return response;
                }
            });
        } catch (ERVException_Exception e) {
            throw new RuntimeException(e);
        }

        return this;
    }

    public MockErvNachrichtPortBuilder withNachrichtAnzahl(final int anzahl) {
        try {
            when(this.mock.nachrichtAnzahl(any())).then(new Answer<NachrichtAnzahlResponse>() {
                @Override
                public NachrichtAnzahlResponse answer(InvocationOnMock invocation) throws Throwable {
                    NachrichtAnzahlResponse response = SchemaUtil.OF_SERVICE.createNachrichtAnzahlResponse();
                    response.setAnzahl(anzahl);

                    SchemaUtil.printWithHeader("NachrichtAnzahl Response", response);

                    return response;
                }
            });
        } catch (ERVException_Exception e) {
            throw new RuntimeException(e);
        }

        return this;
    }
}
