package at.gv.brz.bjujq.erv.samples.common;

import at.gv.justiz.erv.ervnachrichtextern.v3_0.ERVNachrichtTyp;
import at.gv.justiz.erv.ervnachrichtextern.v3_0.EmpfaengerTyp;
import at.gv.justiz.erv.ervnachrichtextern.v3_0.SenderTyp;
import at.gv.justiz.erv.ervnachrichtextern.v3_0.StatusTyp;
import at.gv.justiz.erv.ervnachrichtserviceextern.v3_0.ObjectFactory;
import at.gv.justiz.erv.ervtypenextern.v3_0.AnhangTyp;
import at.gv.justiz.erv.ervtypenextern.v3_0.TeilnehmerKennungTyp;
import at.gv.justiz.erv.ervwertebereichextern.v3_0.*;
import org.w3c.dom.Document;

import javax.activation.DataHandler;
import javax.xml.XMLConstants;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import java.io.ByteArrayOutputStream;
import java.math.BigInteger;
import java.net.URI;
import java.net.URISyntaxException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.UUID;

/**
 * Utility-Methoden fuer Schema-Klassen.
 */
public final class SchemaUtil {

    private static final Schema SCHEMA = initSchema();
    public static final ObjectFactory OF_SERVICE = new ObjectFactory();
    public static final at.gv.justiz.erv.ervnachrichtextern.v3_0.ObjectFactory OF_NR = new at.gv.justiz.erv.ervnachrichtextern.v3_0.ObjectFactory();
    public static final at.gv.justiz.erv.ervtypenextern.v3_0.ObjectFactory OF_TYP = new at.gv.justiz.erv.ervtypenextern.v3_0.ObjectFactory();
    public static final at.gv.justiz.erv.ervanschriftcodeserviceextern.v3_0.ObjectFactory OF_AN_SERVICE = new at.gv.justiz.erv.ervanschriftcodeserviceextern.v3_0.ObjectFactory();
    public static final at.gv.justiz.erv.ervanschriftcodenachrichtextern.v3_0.ObjectFactory OF_AN_NR = new at.gv.justiz.erv.ervanschriftcodenachrichtextern.v3_0.ObjectFactory();
    public static final String SERVICEPROVIDER_TEST = "TEST";
    public static final String SERVICEPROVIDER_JUSTIZ_AT = "JUSTIZ_AT";


    /**
     * Erstelle eine ERV-Nachricht mit Sender Anschriftcode und Empfaenger Justizanwendung.
     *
     * @param anschriftcode Anschriftcode als Sender
     * @param justizanwendung Justizanwendung als Empfaenger
     * @return ERVNachrichtTyp
     */
    public static ERVNachrichtTyp createErvNachrichtToJustizAT(final String anschriftcode, final String justizanwendung) {
        ERVNachrichtTyp ervnachricht = OF_NR.createERVNachrichtTyp();
        ervnachricht.setNachrichtId(ERVUtil.generateRandomId());
        ervnachricht.setErstellzeitpunkt(createCurrentDateTime());

        // Empfaenger ist eine Justizanwendung
        EmpfaengerTyp empfaengerTyp = OF_NR.createEmpfaengerTyp();
        ervnachricht.setEmpfaenger(empfaengerTyp);
        empfaengerTyp.setTeilnehmer(createTeilnehmerJustizanwendung(justizanwendung));
        empfaengerTyp.setServiceArt(Service.JUSTIZ_AT);

        // Sender ist ein Anschriftcode
        SenderTyp senderTyp = OF_NR.createSenderTyp();
        ervnachricht.setSender(senderTyp);
        senderTyp.setTeilnehmer(createTeilnehmerAnschriftcode(anschriftcode));
        senderTyp.setService(OF_TYP.createServiceTyp());
        senderTyp.getService().setServiceArt(Service.UEBERMITTLUNGSSTELLE);
        senderTyp.getService().setServiceProvider(SERVICEPROVIDER_TEST);

        ervnachricht.setZustellqualitaet(Zustellqualitaet.ERV);

        ervnachricht.setNachricht(OF_NR.createNachrichtTyp());
        ervnachricht.getNachricht().setEinbringungszeitpunkt(createCurrentDateTime());
        // Es wird ein Nachweis fuer die Nachricht erwartet
        ervnachricht.getNachricht().setNachweis(true);

        // Erstelle zwei PDF-Dokumente
        // Die Anhang-ID und Reihung von Anhaengen eindeutig deklarieren, vernuenftigerweise von 1 beginnend
        ervnachricht.getAnhang().add(createAnhangPdf("1", 1));
        ervnachricht.getAnhang().add(createAnhangPdf("2", 2));

        validate(OF_NR.createERVNachricht(ervnachricht));
        return ervnachricht;
    }

    /**
     * Erstelle eine ERV-Nachricht mit Sender Justizanwendung und Empfaenger Anschriftcode.
     *
     * @param anschriftcode Anschriftcode als Sender
     * @param justizanwendung Justizanwendung als Empfaenger
     * @return ERVNachrichtTyp
     */
    public static ERVNachrichtTyp createErvNachrichtFromJustizAT(final String anschriftcode, final String justizanwendung) {
        ERVNachrichtTyp ervnachricht = OF_NR.createERVNachrichtTyp();
        ervnachricht.setNachrichtId(ERVUtil.generateRandomId());
        ervnachricht.setErstellzeitpunkt(createCurrentDateTime());

        // Empfaenger ist ein Anschriftcode
        EmpfaengerTyp empfaengerTyp = OF_NR.createEmpfaengerTyp();
        ervnachricht.setEmpfaenger(empfaengerTyp);
        empfaengerTyp.setTeilnehmer(createTeilnehmerAnschriftcode(anschriftcode));
        empfaengerTyp.setServiceArt(Service.UEBERMITTLUNGSSTELLE);

        // Sender ist eine Justizanwendung
        SenderTyp senderTyp = OF_NR.createSenderTyp();
        ervnachricht.setSender(senderTyp);
        senderTyp.setTeilnehmer(createTeilnehmerJustizanwendung(justizanwendung));
        senderTyp.setService(OF_TYP.createServiceTyp());
        senderTyp.getService().setServiceArt(Service.JUSTIZ_AT);
        senderTyp.getService().setServiceProvider(SERVICEPROVIDER_JUSTIZ_AT);

        ervnachricht.setZustellqualitaet(Zustellqualitaet.ERV);

        ervnachricht.setNachricht(OF_NR.createNachrichtTyp());
        ervnachricht.getNachricht().setEinbringungszeitpunkt(createCurrentDateTime());
        // Es wird ein Nachweis fuer die Nachricht erwartet
        ervnachricht.getNachricht().setNachweis(true);

        // Erstelle zwei PDF-Dokumente
        // Die Anhang-ID und Reihung von Anhaengen eindeutig deklarieren, vernuenftigerweise von 1 beginnend
        ervnachricht.getAnhang().add(createAnhangPdf("1", 1));
        ervnachricht.getAnhang().add(createAnhangPdf("2", 2));

        validate(OF_NR.createERVNachricht(ervnachricht));
        return ervnachricht;
    }

    /**
     * Erstelle einen ERV-Nachweis mit Sender Anschriftcode und Empfaenger Justizanwendung.
     *
     * @param anschriftcode Anschriftcode als Sender
     * @param justizanwendung Justizanwendung als Empfaenger
     * @param nachrichtId ERV-ID der zuvor eingebrachten ERV-Nachricht
     * @return ERVNachrichtTyp
     */
    public static ERVNachrichtTyp createErvNachweisToJustizAT(final String anschriftcode, final String justizanwendung, final String nachrichtId) {
        ERVNachrichtTyp ervnachricht = OF_NR.createERVNachrichtTyp();
        ervnachricht.setNachrichtId(ERVUtil.generateRandomId());
        ervnachricht.setErstellzeitpunkt(createCurrentDateTime());

        // Empfaenger ist eine Justizanwendung
        EmpfaengerTyp empfaengerTyp = OF_NR.createEmpfaengerTyp();
        ervnachricht.setEmpfaenger(empfaengerTyp);
        empfaengerTyp.setTeilnehmer(createTeilnehmerJustizanwendung(justizanwendung));
        empfaengerTyp.setServiceArt(Service.JUSTIZ_AT);

        // Sender ist ein Anschriftcode
        SenderTyp senderTyp = OF_NR.createSenderTyp();
        ervnachricht.setSender(senderTyp);
        senderTyp.setTeilnehmer(createTeilnehmerAnschriftcode(anschriftcode));
        senderTyp.setService(OF_TYP.createServiceTyp());
        senderTyp.getService().setServiceArt(Service.UEBERMITTLUNGSSTELLE);
        senderTyp.getService().setServiceProvider(SERVICEPROVIDER_TEST);

        ervnachricht.setZustellqualitaet(Zustellqualitaet.ERV);

        ervnachricht.setNachweis(OF_NR.createNachweisTyp());
        ervnachricht.getNachweis().setNachweisart(NachweisArt.POSITIV);
        ervnachricht.getNachweis().setNachweiszeitpunkt(createCurrentDateTime());
        ervnachricht.getNachweis().setReferenzNachrichtId(nachrichtId);

        // Erstelle zwei PDF-Dokumente
        // Die Anhang-ID und Reihung von Anhaengen eindeutig deklarieren, vernuenftigerweise von 1 beginnend
        ervnachricht.getAnhang().add(createAnhangPdf("1", 1));
        ervnachricht.getAnhang().add(createAnhangPdf("2", 2));

        validate(OF_NR.createERVNachricht(ervnachricht));
        return ervnachricht;
    }

    /**
     * Erstelle einen ERV-Nachweis mit Sender Justizanwendung und Empfaenger Anschriftcode.
     *
     * @param anschriftcode Anschriftcode als Empfaenger
     * @param justizanwendung Justizanwendung als Sender
     * @param nachrichtId ERV-ID der zuvor eingebrachten ERV-Nachricht
     * @return ERVNachrichtTyp
     */
    public static ERVNachrichtTyp createErvNachweisFromJustizAT(final String anschriftcode, final String justizanwendung, final String nachrichtId) {
        ERVNachrichtTyp ervnachricht = OF_NR.createERVNachrichtTyp();
        ervnachricht.setNachrichtId(ERVUtil.generateRandomId());
        ervnachricht.setErstellzeitpunkt(createCurrentDateTime());

        // Empfaenger ist ein Anschriftcode
        EmpfaengerTyp empfaengerTyp = OF_NR.createEmpfaengerTyp();
        ervnachricht.setEmpfaenger(empfaengerTyp);
        empfaengerTyp.setTeilnehmer(createTeilnehmerAnschriftcode(anschriftcode));
        empfaengerTyp.setServiceArt(Service.UEBERMITTLUNGSSTELLE);

        // Sender ist eine Justizanwendung
        SenderTyp senderTyp = OF_NR.createSenderTyp();
        ervnachricht.setSender(senderTyp);
        senderTyp.setTeilnehmer(createTeilnehmerJustizanwendung(justizanwendung));
        senderTyp.setService(OF_TYP.createServiceTyp());
        senderTyp.getService().setServiceArt(Service.JUSTIZ_AT);
        senderTyp.getService().setServiceProvider(SERVICEPROVIDER_JUSTIZ_AT);

        ervnachricht.setZustellqualitaet(Zustellqualitaet.ERV);

        ervnachricht.setNachweis(OF_NR.createNachweisTyp());
        ervnachricht.getNachweis().setNachweisart(NachweisArt.POSITIV);
        ervnachricht.getNachweis().setNachweiszeitpunkt(createCurrentDateTime());
        ervnachricht.getNachweis().setReferenzNachrichtId(nachrichtId);


        // Erstelle zwei PDF-Dokumente
        // Die Anhang-ID und Reihung von Anhaengen eindeutig deklarieren, vernuenftigerweise von 1 beginnend
        ervnachricht.getAnhang().add(createAnhangPdf("1", 1));
        ervnachricht.getAnhang().add(createAnhangPdf("2", 2));

        validate(OF_NR.createERVNachricht(ervnachricht));
        return ervnachricht;
    }

    /**
     * Erstelle einen PDF-Anhang mit ID <code>id</code> und Reihung <code>reihung</code>.
     *
     * @param id String
     * @param reihung int
     * @return AnhangTyp
     */
    public static AnhangTyp createAnhangPdf(final String id, final int reihung) {
        AnhangTyp anhangTyp = OF_TYP.createAnhangTyp();
        anhangTyp.setAnhangId(id);
        anhangTyp.setReihung(BigInteger.valueOf(reihung));
        anhangTyp.setArt(AnhangArt.DOKUMENT);

        final byte[] content = "PDF Inhalt".getBytes();
        anhangTyp.setContentType(ContentType.APPLICATION_PDF);
        anhangTyp.setContent(new DataHandler(new ByteArrayDataSource(content, "PDF-Name", "application/pdf")));

        anhangTyp.setMetadaten("Metadaten des PDF-Dokuments".getBytes());

        anhangTyp.setHash(OF_TYP.createHashTyp());
        anhangTyp.getHash().setHashAlgorithmus(HashAlgorithmus.MD_5);
        anhangTyp.getHash().setHashWert(calcMD5Hex(content));

        return anhangTyp;
    }

    /**
     * Erstelle einen Teilnehmer mit Justizanwendung <code>justizanwendung</code>.
     *
     * @param justizanwendung String
     * @return TeilnehmerKennungTyp
     */
    public static TeilnehmerKennungTyp createTeilnehmerJustizanwendung(final String justizanwendung) {
        TeilnehmerKennungTyp teilnehmerKennungTyp = OF_TYP.createTeilnehmerKennungTyp();
        teilnehmerKennungTyp.setTeilnehmerKennung(OF_TYP.createKennungTyp());
        teilnehmerKennungTyp.getTeilnehmerKennung().setJustizanwendung(justizanwendung);
        return teilnehmerKennungTyp;
    }

    /**
     * Erstelle einen Teilnehmer mit Anschriftcode <code>anschriftcode</code>.
     *
     * @param anschriftcode String
     * @return TeilnehmerKennungTyp
     */
    public static TeilnehmerKennungTyp createTeilnehmerAnschriftcode(final String anschriftcode) {
        TeilnehmerKennungTyp teilnehmerKennungTyp = OF_TYP.createTeilnehmerKennungTyp();
        teilnehmerKennungTyp.setTeilnehmerKennung(OF_TYP.createKennungTyp());
        teilnehmerKennungTyp.getTeilnehmerKennung().setAnschriftcode(anschriftcode);
        return teilnehmerKennungTyp;
    }

    /**
     *  Erstelle einen Status mit Status-Code <code>statuscode</code>.
     *
     * @param statuscode NachrichtStatuscode
     * @return StatusTyp
     */
    public static StatusTyp createStatus(NachrichtStatuscode statuscode) {
        StatusTyp statusTyp = OF_NR.createStatusTyp();
        statusTyp.setCode(statuscode);
        statusTyp.setZeitpunkt(createCurrentDateTime());
        return statusTyp;
    }

    /**
     * Erstelle eine XML Zeit-Objekt.
     *
     * @return XMLGregorianCalendar
     */
    public static XMLGregorianCalendar createCurrentDateTime() {
        try {
            return DatatypeFactory.newInstance().newXMLGregorianCalendar(new GregorianCalendar());
        } catch (DatatypeConfigurationException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * Erstelle ein leeres DOM-Element.
     *
     * @return Element
     */
    public static org.w3c.dom.Element createEmptyElement() {
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = dbf.newDocumentBuilder();
            Document doc = builder.newDocument();
            return doc.createElement("dummy");
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * Berechne den HEX-MD5-Wert.
     *
     * @param data Eingabedaten
     * @return String
     */
    public static String calcMD5Hex(final byte[] data) {
        MessageDigest md5;
        try {
            md5 = MessageDigest.getInstance("MD5");
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        }

        md5.update(data);
        StringBuilder hex = new StringBuilder();

        for (byte b : md5.digest()) {
            String hexValue = Integer.toHexString(b & 0xFF);
            if (hexValue.length() == 2) {
                hex.append(hexValue);
            } else {
                hex.append("0").append(hexValue);
            }
        }

        return hex.toString();
    }

    /**
     * Validiere das Objekt gegen das ERV-Schema.
     *
     * @param obj Objekt einer generierten Klasse
     * @param <T> Typ des Objektes
     */
    public static <T> void validate(final T obj) {
        JAXBContext ctx = null;
        Marshaller marshaller = null;
        ByteArrayOutputStream result = new ByteArrayOutputStream();
        try {
            ctx = JAXBContext.newInstance(targetObjectClass(obj));
            marshaller = ctx.createMarshaller();
            marshaller.setSchema(SCHEMA);
            marshaller.setEventHandler(new JaxbValidationEventHandler());
            marshaller.marshal(obj, result);
        } catch (JAXBException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * Ausgabe des Objektes als XML.
     *
     * Zusaetzlich wird eine Kopfzeile mit ausgegeben.
     *
     * @param header Kopfzeile die ausgegeben werden soll
     * @param obj Objekt welches als XML ausgegeben werden soll
     * @param <T> Typ des Objektes
     */
    public static <T> void printWithHeader(final String header, final T obj) {
        System.out.println("-------------- " + header + " ---------------");
        print(obj);
        System.out.println("");
    }

    /**
     * Ausgabe des Objektes als XML.
     *
     * @param obj Objekt welches als XMl ausgegeben werden soll
     * @param <T> Typ des Objektes
     */
    public static <T> void print(final T obj) {
        JAXBContext ctx = null;
        Marshaller marshaller = null;
        ByteArrayOutputStream result = new ByteArrayOutputStream();
        try {
            ctx = JAXBContext.newInstance(targetObjectClass(obj));
            marshaller = ctx.createMarshaller();
            marshaller.marshal(obj, result);
            System.out.println(new String(result.toByteArray()));
        } catch (JAXBException e) {
            throw new RuntimeException(e);
        }
    }

    private static Schema initSchema() {
        SchemaFactory schemaFactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
        try {
            Source[] sources = Arrays.stream(new String[] {"xsd/ERVNachrichtExtern.xsd", "xsd/ERVAnschriftcodeNachrichtExtern.xsd"})
                    .map(path -> toUri(path))
                    .map(uri -> new StreamSource(uri.toString()))
                    .toArray(Source[]::new);

            return schemaFactory.newSchema(sources);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private static URI toUri(final String path) {
        try {
            return Thread.currentThread().getContextClassLoader().getResource(path).toURI();
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }

    private static Class<?> targetObjectClass(Object obj) {
        if (obj instanceof JAXBElement) {
            JAXBElement jaxbElement = (JAXBElement) obj;
            return jaxbElement.getValue().getClass();
        } else {
            return obj.getClass();
        }
    }

    private static String targetObjectClassName(Object obj) {
        Class<?> clazz = targetObjectClass(obj);
        return clazz.getName();
    }

    private SchemaUtil() {

    }
}
