package at.gv.brz.bjujq.erv.samples.nachricht_nachweis;

import at.gv.brz.bjujq.erv.samples.common.MockErvNachrichtPortBuilder;
import at.gv.brz.bjujq.erv.samples.common.SchemaUtil;
import at.gv.justiz.erv.ervnachrichtextern.v3_0.ERVNachrichtTyp;
import at.gv.justiz.erv.ervnachrichtextern.v3_0.StatusTyp;
import at.gv.justiz.erv.ervnachrichtserviceextern.v3_0.*;
import at.gv.justiz.erv.ervwertebereichextern.v3_0.NachrichtStatuscode;

public class NachrichtAbholen {

    /**
     * Die Abholung von Nachrichten besteht grundsätzlich aus drei Schritten:
     *
     * <ul>
     *     <li>Anzahl der abholbereiten Nachrichten abfragen</li>
     *     <li>Nachrichten abholen</li>
     *     <li>Nachrichten bestaetigen</li>
     * </ul>
     *
     */
    public static void main(String[] args) throws ERVException_Exception {
        ERVNachrichtPortType ervNachrichtPortType = new MockErvNachrichtPortBuilder()
                                .withNachrichtAnzahl(1)
                                .withNachrichtAbholen("RXXXXXX", "VJ")
                                .withNachrichtBestaetigen()
                                .build();

        // Ermittlung der Nachrichten die grundsätzlich zur Abholung bereitstehen
        NachrichtAnzahlResponse nachrichtAnzahlResponse = ervNachrichtPortType.nachrichtAnzahl(SchemaUtil.createEmptyElement());

        // Abholen der Nachrichten
        for (int i = 0; i < nachrichtAnzahlResponse.getAnzahl(); i++) {

            NachrichtAbholenResponse response = ervNachrichtPortType.nachrichtAbholen(SchemaUtil.createEmptyElement());

            // Fehlercodes siehe ERV-Publikation
            if (response.getFehler() != null) {
                if ("ERV_0043".equals(response.getFehler().getFehlercode())) {
                    // Keine Nachrichten mehr zum Abholen
                   break;
                }
            }

            // Nachdem eine Nachricht vom ERV abgeholt wurde befindet sich diese im Status AUSGEFOLGT
            ERVNachrichtTyp ervNachricht = response.getERVNachricht();

            // Kann die Nachricht weiterverarbeitet werden, so ist diese in weiterer Folge zu bestaetigen
            NachrichtBestaetigenRequest bestaetigenRequest = SchemaUtil.OF_SERVICE.createNachrichtBestaetigenRequest();
            bestaetigenRequest.setNachrichtenkennung(SchemaUtil.OF_NR.createNachrichtenkennungTyp());
            bestaetigenRequest.getNachrichtenkennung().setNachrichtId(ervNachricht.getNachrichtId());

            NachrichtBestaetigenResponse bestaetigenResponse = ervNachrichtPortType.nachrichtBestaetigen(bestaetigenRequest);

            // Fehlercodes siehe ERV-Publikation
            if (bestaetigenResponse.getFehler() != null) {
                // Fehlerbehandlung
            } else {
                StatusTyp status = bestaetigenResponse.getNachrichtstatus();

                if (status.getCode() == NachrichtStatuscode.ZUGESTELLT) {
                    // Nach einer erfolgreichen Bestaetigung ist die Nachricht im Status ZUGESTELLT und damit die Abholung beendet
                }

                // In weiterer Folge kann fuer die bestaetigte Nachricht ein Nachweis erstellt werden (siehe Klasse NachweisEinbringen)
            }
        }
    }

}
